;****************************************************************************
;*	RGB LED-Stripes	Dimmer													*
;*																			*
;*	Mit diesem Programm kann man sog. RGB-LED-Stripes ber eine				*
;*	Fernbedienung (RC5-Code) Dimmen. Zum dekodieren des RC5-Codes wird die 	*
;*	von Atmel zur verfgung gestellte A410 Routine benutzt.					*
;*	Die LEDs werden ber Software-PWM gesteuert und knnen 64 (0..63)		*
;*	Verschiedene Helligkeitsstufen annehmen. Die Geringe Auflsung ist ein	*
;*	Kompromiss zwischen Bedienbarkeit mit einer Fernbedienung und weichen	*
;*	Farbverlufen.Bei einer eine 8-Bit Auflsung (0..255) dauert das		*
;*	dimmen einer Farbe ber FB >30 Sekunden.								*
;* 																			*
;*	Da das Menschliche Auge den Dimmverlauf von 0..100% nicht linear		*
;*	wahrnimmt, habe ich eine Tabelle (mit 8-Bit-Werten)  erstellt, die 		*
;*	das ausgleichen soll. Die Werte sind nach folgender Formel erstellt:	*
;*																			*
;*	Wert(x)= int(255*(x/63)^2.2+0.5) wobei x (1..64) die entsprechende 		*
;*	Platzierung in der Tabelle darstellt.									*
;*	(Quelle http://www.mikrocontroller.net/articles/Diskussion:LED-Fading	*
;*																			*
;*  Version1.0 von 15.11.2008												*
;*	von Jrgen Woetzel														*
;*	www.avr-projekte.de														*
;****************************************************************************
.include "tn2313def.inc"
.device attiny2313


.equ	INPUT		=6	;PD6, da muss der TSOP ran
.equ	SYS_ADDR	=5	;0 ist der TV,5 oder 6 ist der Videorec.

.def	ref1	=R1		;Variable fr Appnote A410
.def	ref2	=R2
.def	timerL	=R16	;8MHz/256
.def	timerH	=R17	;TimerL/256
.def	temp	=R18	;wird von mir mitbenutzt
.def	system	=R19	;Gerteadresse
.def	command	=R20	;Kommando
.def	bitcnt	=R21
;-------------------------------------------------------------

.def	S		=R3		;Status-Byte
.def	pwm		=r22	;Zhler fr ISR	
.def 	rot		=r23	;6bit RGB-Werte
.def	gruen	=r24
.def	blau	=r25
.def	rt		=r4		;8bit RGB-Werte
.def	gn		=r5
.def	bl		=r6
.def	zr		=r26	;Dimmregister (Ziel)
.def	zg		=r27
.def	zb		=r28


.cseg
.org 0
		rjmp	reset
.org OVF0addr
;======================================================================================
;ISR
;---------------------------------------------------------------------------------------
TIM0_OVF:	
		in		S,sreg				
									;TimerL und TimerH werden fr (IR) Detect bentigt
		inc		timerL				;Alle 32 us bei 8 MHz
		brne	RGB
		inc		timerH				;Ca. alle 8 ms (bissle mehr)
;--------------------------------------------------------------------------------------
;	Hier werden die 6Bit Werte (0..63) aus der (dem Auge angepassten)
;	8Bit Tabelle gelesen. Das High-Byte des Z-Pointers  wird im Hauptprogramm gesetzt.
;	Um wertvolle Zyklen zu sparen habe ich die Dimmtabelle nach $300 gelegt- so kann der
;	jeweilige 6Bit RGB-Wert direkt in das Low Byte des Z-Pointers kopiert werden und beim
;	High Byte muss nicht extra addiert werden.
;---------------------------------------------------------------------------------------
RGB:	mov		zl,rot				;Rotwert in Z-Pointer			
		lpm		rt,z				;8-Bit Rotwert aus Tabelle nach rt
		mov		zl,gruen			;usw
		lpm		gn,z
		mov		zl,blau
		lpm		bl,z
;---------------------------------------------------------------------------------------
;	Hier wird der Zhler (PWM 0..255) mit den 8Bit RGB-Werten verglichen und die Farbe 
;	deren Wert geringer ist abgeschaltet (PBx =1, negativ Logik).
;---------------------------------------------------------------------------------------
		cp		pwm,rt				;Rotwert vergleichen
		brlo	pwg					;wenn kleiner..
		sbi		portb,pb0			;..Rot aus
pwg:	cp		pwm,gn				;Grnwert
		brlo	pwb
		sbi		portb,pb1
pwb:	cp		pwm,bl				;und Blau
		brlo	pwex
		sbi		portb,pb2
pwex:	inc		pwm					;Zhler +1
		brne	TIM0_OVF_exit		;Wenn kein berlauf, Ende ISR
;----------------------------------------------------------------------------------------
;Bei berlauf (PWM =0) werden alle Farben eingeschaltet, es sei denn der Wert steht auf 0
;---------------------------------------------------------------------------------------
		tst		rt					;Rotwert testen
		breq	tg					;wenn Wert=0, ausgeschaltet lassen.
		cbi		portb,pb0			;sonst.. Rot "Ein"
tg:		tst		gn					;usw
		breq	tb
		cbi		portb,pb1
tb:		tst		bl
		breq	TIM0_OVF_exit
		cbi		portb,pb2
;---------------------------------------------------------------------------------------

TIM0_OVF_exit:
		out		sreg,S
		reti
;========================================================================================
reset:	ldi  	temp,low(RAMEND)	;Stack
		out		SPL,temp			
;		ldi		temp,high(RAMEND)
;		out		SPH,temp

		ldi		temp,1				;Timer/Counter 0 ohne Vorteiler
		out		TCCR0,temp			;alle 256 Taktzyklen (32us) ein Timerinterrupt	
		ldi		temp,1<<TOIE0		;Timer0 Einschalten
		out		TIMSK,temp

		ser		temp				;PORTB Ausgang
		out		DDRB,temp

		clr		pwm					
		ldi		rot,0				;erst mal alles dunkel
		ldi		gruen,0
		ldi		blau,0
		ldi		zh,high(dimtab*2)	;Z-Pointer High Byte setzen (=6 bei $300)
		sei							;ISR Los
;------------------------------------------------------------------------
;Tasten der Fernbedienung abfragen
main:	rcall	detect				;springt in die AVR410 Routine
		cpi		system,SYS_ADDR		;Welches Gert ?
		brne	main				;ist nicht fr mich
		andi	command,0x3F		;Controll-Bit ausblenden



t0:		cpi		command,54			;Taste Stop auf Fernbedienung dunkel (Aus)
		brne	t1
		rcall	dimdwn
		rjmp	main

t1:		cpi		command,1			;Taste 1 auf Fernbedienung +rot
		brne	t2
		cpi		rot,64
		breq	main
		inc		rot
		rjmp	main

t2:		cpi		command,4			;Taste 4 auf Fernbedienung -rot
		brne	t3
		cpi		rot,0
		breq	main
		dec		rot
		rjmp	main

t3:		cpi		command,2			;Taste 2 auf Fernbedienung +grn
		brne	t4
		cpi		gruen,64
		breq	main
		inc		gruen
		rjmp	main

t4:		cpi		command,5			;Taste 5 auf Fernbedienung -grn
		brne	t5
		cpi		gruen,0
		breq	main
		dec		gruen
		rjmp	main

t5:		cpi		command,3			;Taste 3 auf Fernbedienung +blau
		brne	t6
		cpi		blau,64
		breq	main
		inc		blau
		rjmp	main

t6:		cpi		command,6			;Taste 6 auf Fernbedienung -blau
		brne	t7
		cpi		blau,0
		breq	main
		dec		blau
		rjmp	main


t7:		cpi		command,32			;Taste Channel+ auf Fernbedienung heller -> weiss
		brne	t8
		cpi		rot,64
		breq	hell1
		inc		rot
hell1:	cpi		gruen,64
		breq	hell2
		inc		gruen
hell2:	cpi		blau,64
		breq	main
		inc		blau
		rjmp	main

t8:		cpi		command,33		;Taste Channel- auf Fernbedienung dunkler
		brne	t9
		tst		rot
		breq	dunk1
		dec		rot
dunk1:	tst		gruen
		breq	dunk2
		dec		gruen
dunk2:	tst		blau
		breq	main1
		dec		blau
		rjmp	main



t9:		cpi		command,0x07		;Taste 7 auf Fernbedienung weiss 66%
		brne	t10
		ldi		zr,43
		ldi		zg,43
		ldi		zb,43
		rcall	dimm
		rjmp	main
	
t10:	cpi		command,0x08		;Taste 8 auf Fernbedienung weiss 100%
		brne	t11
		ldi		zr,63
		ldi		zg,63
		ldi		zb,63
		rcall	dimm
		rjmp	main
	
t11:	cpi		command,0x09		;Taste 9 auf Fernbedienung Gelb
		brne	t12
		ldi		zr,44				;
		ldi		zg,44
		ldi		zb,0
		rcall	dimm
		rjmp	main

t12:	cpi		command,53			;Taste Play auf Fernbedienung 
		brne	t13
		rjmp	mood


t13:	cpi		command,0			;Taste 0 auf Fernbedienung Hellblau
		brne	t14
		ldi		zr,20
		ldi		zg,20
		ldi		zb,64
		rcall	dimm
		rjmp	main


t14:
main1:	rjmp	main



mood:	clr		rot
		clr		gruen
		clr		blau

m1:		inc		blau			;blau +
		rcall	wait1
		brts	main1
		cpi		blau,64
		brne	m1

m2:		inc		rot				;rot+, blau an
		rcall	wait
		brts	main1
		cpi		rot,64
		brne	m2

m3:		dec		blau			;blau-
		rcall	wait
		brts	main1
		tst		blau
		brne	m3

m4:		dec		rot				;grn+, rot-
		inc		gruen
		rcall	wait
		brts	main1
		cpi		gruen,64
		brne	m4

m5:		inc		blau			;blau+ grn an
		rcall	wait
		brts	main1
		cpi		blau,64
		brne	m5


m6:		dec		gruen			;grn- blau an
		rcall	wait
		brts	main1
		tst		gruen
		brne	m6


		rjmp	m2

stop:	rjmp	stop



wait:	rcall	detect				;springt in die AVR410 Routine
		clt
		cpi		system,SYS_ADDR		;Welches Gert ?
		brne	notast				;ist nicht fr mich
		andi	command,0x3F		;Controll-Bit ausblenden
;		cpi		command,54			;Taste Stop ?
;		brne	notast
		set
notast:	ret


dimup:	cp		rot,temp
		breq	du1
		inc		rot
		inc		gruen
		inc		blau
		rcall	wait1
		rjmp	dimup
du1:	ret




dimdwn:	tst		rot
		breq	dd1
		dec		rot
dd1:	tst		gruen
		breq	dd2
		dec		gruen
dd2:	tst		blau
		breq	dd3
		dec		blau
			
dd3:	rcall	wait1
		clr		temp
		add		temp,rot
		add		temp,gruen
		add		temp,blau
		brne	dimdwn
		ret
;---------------------------------------

dimm:	cp		rot,zr
		brlo	rup
		breq	dimg
		dec		rot
		rjmp	dimg
rup:	inc		rot

dimg:	cp		gruen,zg
		brlo	gup
		breq	dimb
		dec		gruen
		rjmp	dimb
gup:	inc		gruen

dimb:	cp		blau,zb
		brlo	bup
		breq	dim1
		dec		blau
		rjmp	dim1
bup:	inc		blau

dim1:	rcall	wait1
		cp		zr,rot
		brne	dimm
		cp		zg,gruen
		brne	dimm
		cp		zb,blau
		brne	dimm		
		

exdim:	ret
;-------------------------------------------

wait1:	clr		timerh
		clr		timerl
w1:		cpi		timerh,3
		brne	w1
		ret




;********************************************************************
;*
;* Ab hier folgt die RC5 Decoderroutine von Atmel "Appnote AVR410",
;* fast 1:1 bernommen.
;* Die Routine luft jetzt auf einem Tiny2313 mit 8 MHz . Der interne
;* R/C Oszilator gengt, da die lnge des das ersten Startbits gemessen wird.
;* (Genial ^^)
;*
;********************************************************************


detect:	clr		timerH
detect1:clr		timerL
detect2:cpi		timerH,16		;8		;If line not idle within 131ms
		brlo	dl1
		rjmp	fault			;  then exit
	
dl1:	cpi		timerL,109		;55		;If line low for 3.5ms
		brge	start1			;   then wait for start bit
		sbis	PIND,INPUT		;If line is
		rjmp	detect1			;   low  - jump to detect1
		rjmp	detect2			;   high - jump to detect2

start1:	cpi		timerH,16		;	8		;If no start bit detected
		brge	fault			;within 130ms then exit
		sbic	PIND,INPUT		;Wait for start bit
		rjmp	start1
		clr		timerL			;Measure length of start bit
		
start2:	cpi		timerL,34		;17		;If startbit longer than 1.1ms,
		brge	fault			;   exit
		sbis	PIND,INPUT
		rjmp	start2
								;Positive edge of 1st start bit
		mov		temp,timerL		;timer is 1/2 bit time
		clr		timerL
		mov		ref1,temp
		lsr		ref1
		mov		ref2,ref1
		add		ref1,temp		;ref1 = 3/4 bit time
		lsl		temp
		add		ref2,temp		;ref2 = 5/4 bit time
	
start3:	cp		timerL,ref1		;If high periode St2 > 3/4 bit time
		brge	fault			;   exit
		sbic	PIND,INPUT		;Wait for falling edge start bit 2
		rjmp	start3
		clr		timerL
		ldi		bitcnt,12		;Receive 12 bits
		clr		command
		clr		system

sample:	cp		timerL,ref1		;Sample INPUT at 1/4 bit time
		brlo	sample
		sbic	PIND,INPUT
		rjmp	bit_is_a_1		;Jump if line high

bit_is_a_0:
		clc						;Store a '0'
		rol		command
		rol		system

								;Synchronize timing
bit_is_a_0a:
		cp		timerL,ref2	;If no edge within 3/4 bit time
		brge	fault			;   exit
		sbis	PIND,INPUT		;Wait for rising edge
		rjmp	bit_is_a_0a		;in the middle of the bit

		clr		timerL
		rjmp	nextbit

bit_is_a_1:
		sec						;Store a '1'
		rol	command
		rol	system
								;Synchronize timing
bit_is_a_1a:
		cp	timerL,ref2			;If no edge within 3/4 bit time
		brge	fault			;   exit
		sbic	PIND,INPUT		;Wait for falling edge
		rjmp	bit_is_a_1a		;in the middle of the bit
		clr		timerL

nextbit:dec		bitcnt			;If bitcnt > 0
		brne	sample			;   get next bit


;All bits sucessfully received!

		mov		temp,command	;Place system bits in "system"
		rol		temp
		rol		system
		rol		temp
		rol		system
		bst		system,5		;Move toggle bit
		bld		command,6		;to "command"
								;Clear remaining bits
		andi	command,0b01111111
		andi	system,0x1F
		ret

fault:	ser		command			;Both "command" and "system"
		ser		system			;0xFF indicates failure
		ret

;tabelle
.org	$300					;Tab an $300
dimtab:
.db	  0,  1,  1,  1,  1,  2,  2,  2
.db	  3,  4,  5,  6,  7,  8, 10, 11
.db	 13, 15, 17, 19, 21, 23, 26, 28
.db	 31, 34, 37, 40, 43, 47, 50, 54
.db	 58, 62, 66, 71, 75, 80, 84, 89
.db	 94,100,105,111,116,122,128,134
.db	141,147,154,161,168,175,182,190
.db	197,205,213,221,230,238,247,255


